@@ -2,13 +2,13 @@
  *
  * Translate algorithm identifiers (or names) to algorithm objects.
  *
- * $Id: algorithms.h,v 1.9 2000/11/26 17:12:04 nisse Exp $ */
+ */
 
 #ifndef LSH_ALGORITHMS_H_INCLUDED
 #define LSH_ALGORITHMS_H_INCLUDED
 
-#include "abstract_compress.h"
 #include "alist.h"
+#include "compress.h"
 #include "lsh_argp.h"
 #include "randomness.h"
 
