@@ -32,6 +32,7 @@
 
 #include "format.h"
 #include "io.h"
+#include "lsh_string.h"
 #include "read_data.h"
 #include "ssh.h"
 #include "werror.h"
@@ -132,10 +133,10 @@ format_open_confirmation(struct ssh_channel *channel,
 
   packet = lsh_string_alloc(l1 + l2);
 
-  ssh_format_write(CONFIRM_FORMAT, l1, packet->data, CONFIRM_ARGS);
+  ssh_format_write(CONFIRM_FORMAT, packet, 0, CONFIRM_ARGS);
 
   va_start(args, format);
-  ssh_vformat_write(format, l2, packet->data+l1, args);
+  ssh_vformat_write(format, packet, l1, args);
   va_end(args);
 
   return packet;
@@ -460,8 +461,8 @@ adjust_rec_window(struct flow_controlled *f, uint32_t written)
    * may live longer than the actual channel. */
   if (! (channel->flags & (CHANNEL_RECEIVED_EOF | CHANNEL_RECEIVED_CLOSE
 			   | CHANNEL_SENT_CLOSE)))
-    C_WRITE(channel->connection,
-	    prepare_window_adjust(channel, written));
+    connection_send(channel->connection,
+		    prepare_window_adjust(channel, written));
 }
 
 void
@@ -469,9 +470,9 @@ channel_start_receive(struct ssh_channel *channel,
 		      uint32_t initial_window_size)
 {
   if (channel->rec_window_size < initial_window_size)
-    C_WRITE(channel->connection,
-	    prepare_window_adjust
-	    (channel, initial_window_size - channel->rec_window_size));
+    connection_send(channel->connection,
+		    prepare_window_adjust
+		    (channel, initial_window_size - channel->rec_window_size));
 }
 
 /* Channel related messages */
@@ -516,10 +517,10 @@ send_global_request_responses(struct ssh_connection *connection,
  
       object_queue_remove_head(q);
 
-      C_WRITE(connection,
-	      (n->status
-	       ? format_global_success()
-	       : format_global_failure()));
+      connection_send(connection,
+		      (n->status
+		       ? format_global_success()
+		       : format_global_failure()));
     }
 }
 
@@ -605,7 +606,7 @@ DEFINE_PACKET_HANDLER(static, global_request_handler, connection, packet)
   int name;
   int want_reply;
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_GLOBAL_REQUEST)
@@ -625,7 +626,7 @@ DEFINE_PACKET_HANDLER(static, global_request_handler, connection, packet)
 	}
       if (!req)
 	{
-	  C_WRITE(connection, format_global_failure());
+	  connection_send(connection, format_global_failure());
 	  return;
 	}
       else
@@ -660,13 +661,13 @@ DEFINE_PACKET_HANDLER(static, global_request_handler, connection, packet)
 DEFINE_PACKET_HANDLER(static, global_success_handler,
 		      connection, packet)
 {
-  if (packet->length != 1)
+  if (lsh_string_length(packet) != 1)
     {
       PROTOCOL_ERROR(connection->e, "Invalid GLOBAL_REQUEST_SUCCESS message.");
       return;
     }
 
-  assert(packet->data[0] == SSH_MSG_REQUEST_SUCCESS);
+  assert(lsh_string_data(packet)[0] == SSH_MSG_REQUEST_SUCCESS);
 
   if (object_queue_is_empty(&connection->table->pending_global_requests))
     {
@@ -686,13 +687,13 @@ STATIC_EXCEPTION(EXC_GLOBAL_REQUEST, "Global request failed");
 DEFINE_PACKET_HANDLER(static, global_failure_handler,
 		      connection, packet)
 {
-  if (packet->length != 1)
+  if (lsh_string_length(packet) != 1)
     {
       PROTOCOL_ERROR(connection->e, "Invalid GLOBAL_REQUEST_FAILURE message.");
       return;
     }
 
-  assert(packet->data[0] == SSH_MSG_REQUEST_FAILURE);
+  assert(lsh_string_data(packet)[0] == SSH_MSG_REQUEST_FAILURE);
 
   if (object_queue_is_empty(&connection->table->pending_global_requests))
     {
@@ -708,7 +709,11 @@ DEFINE_PACKET_HANDLER(static, global_failure_handler,
 
 /* FIXME: Don't store the channel here, instead have it passed as the
  * argument of the continuation. This might also allow some
- * unification with the handling of global_requests. */
+ * unification with the handling of global_requests.
+ *
+ * This won't quite work yet, because not all channel request
+ * handlers, in particular gateway_channel_request and
+ * x11_req_handler, return the channel in question. */
 
 /* GABA:
    (class
@@ -731,10 +736,10 @@ send_channel_request_responses(struct ssh_channel *channel,
 
       object_queue_remove_head(q);
 
-      C_WRITE(channel->connection,
-	      (n->status
-	       ? format_channel_success(channel->channel_number)
-	       : format_channel_failure(channel->channel_number)));
+      connection_send(channel->connection,
+		      (n->status
+		       ? format_channel_success(channel->channel_number)
+		       : format_channel_failure(channel->channel_number)));
     }
 }
 
@@ -842,7 +847,7 @@ DEFINE_PACKET_HANDLER(static, channel_request_handler,
   struct channel_request_info info;
   uint32_t channel_number;
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_channel_request(&buffer, &channel_number, &info))
     {
@@ -897,8 +902,8 @@ DEFINE_PACKET_HANDLER(static, channel_request_handler,
 	  else
 	    {
 	      if (info.want_reply)
-		C_WRITE(connection,
-			format_channel_failure(channel->channel_number));
+		connection_send(connection,
+				format_channel_failure(channel->channel_number));
 	    }
 	}
       else
@@ -949,8 +954,9 @@ do_channel_open_continue(struct command_continuation *c,
   /* FIXME: Doesn't support sending extra arguments with the
    * confirmation message. */
 
-  C_WRITE(self->connection,
-	  format_open_confirmation(channel, self->local_channel_number, ""));
+  connection_send(self->connection,
+		  format_open_confirmation(channel,
+					   self->local_channel_number, ""));
 }
 
 static struct command_continuation *
@@ -1000,9 +1006,9 @@ do_exc_channel_open_handler(struct exception_handler *s,
 
 	dealloc_channel(table, self->local_channel_number);
 	
-        C_WRITE(self->connection,
-		format_open_failure(self->remote_channel_number,
-				    exc->error_code, e->msg, ""));
+        connection_send(self->connection,
+			format_open_failure(self->remote_channel_number,
+					    exc->error_code, e->msg, ""));
 	break;
       }
     default:
@@ -1065,7 +1071,7 @@ DEFINE_PACKET_HANDLER(static, channel_open_handler,
   struct simple_buffer buffer;
   struct channel_open_info info;
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_channel_open(&buffer, &info))
     {
@@ -1078,10 +1084,11 @@ DEFINE_PACKET_HANDLER(static, channel_open_handler,
 	{
 	  /* We are waiting for channels to close. Don't open any new ones. */
 
-	  C_WRITE(connection,
-		  format_open_failure(info.remote_channel_number,
-				      SSH_OPEN_ADMINISTRATIVELY_PROHIBITED,
-				      "Waiting for channels to close.", ""));
+	  connection_send(connection,
+			  format_open_failure
+			    (info.remote_channel_number,
+			     SSH_OPEN_ADMINISTRATIVELY_PROHIBITED,
+			     "Waiting for channels to close.", ""));
 	}
       else
 	{
@@ -1098,21 +1105,26 @@ DEFINE_PACKET_HANDLER(static, channel_open_handler,
 	  
 	  if (!open)
 	    {
-	      C_WRITE(connection,
-		      format_open_failure(info.remote_channel_number,
-					  SSH_OPEN_UNKNOWN_CHANNEL_TYPE,
-					  "Unknown channel type", ""));
+	      connection_send(connection,
+			      format_open_failure
+			        (info.remote_channel_number,
+				 SSH_OPEN_UNKNOWN_CHANNEL_TYPE,
+				 "Unknown channel type", ""));
 	    }
 	  else
 	    {
 	      int local_number = alloc_channel(connection->table);
 
 	      if (local_number < 0)
-		C_WRITE(connection,
-			format_open_failure(info.remote_channel_number,
-					    SSH_OPEN_RESOURCE_SHORTAGE,
-					    "Channel limit exceeded.", ""));
-
+		{
+		  connection_send(connection,
+				  format_open_failure
+				    (info.remote_channel_number,
+				     SSH_OPEN_RESOURCE_SHORTAGE,
+				     "Channel limit exceeded.", ""));
+		  return;
+		}
+	      
 	      CHANNEL_OPEN(open, connection,
 			   &info,
 			   &buffer,
@@ -1142,7 +1154,7 @@ DEFINE_PACKET_HANDLER(static, window_adjust_handler,
   uint32_t channel_number;
   uint32_t size;
 
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_CHANNEL_WINDOW_ADJUST)
@@ -1159,11 +1171,9 @@ DEFINE_PACKET_HANDLER(static, window_adjust_handler,
 	  if (! (channel->flags & (CHANNEL_SENT_CLOSE | CHANNEL_SENT_EOF)))
 	    {
 	      channel->send_window_size += size;
+
 	      if (channel->send_window_size && channel->send_adjust)
-		{
-		  assert(channel->send_window_size);
-		  CHANNEL_SEND_ADJUST(channel, size);
-		}
+		CHANNEL_SEND_ADJUST(channel, size);
 	    }
 	}
       else
@@ -1185,7 +1195,7 @@ DEFINE_PACKET_HANDLER(static, channel_data_handler,
   uint32_t channel_number;
   struct lsh_string *data;
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_CHANNEL_DATA)
@@ -1208,29 +1218,30 @@ DEFINE_PACKET_HANDLER(static, channel_data_handler,
 	    }
 	  else
 	    {
-              if (data->length > channel->rec_max_packet)
+	      uint32_t length = lsh_string_length(data);
+              if (length > channel->rec_max_packet)
                 {
                   werror("Channel data larger than rec_max_packet. Extra data ignored.\n");
 		  lsh_string_trunc(data, channel->rec_max_packet);
                 }
 
-	      if (data->length > channel->rec_window_size)
+	      if (length > channel->rec_window_size)
 		{
 		  /* Truncate data to fit window */
 		  werror("Channel data overflow. Extra data ignored.\n");
 		  debug("   (data->length=%i, rec_window_size=%i).\n", 
-			data->length, channel->rec_window_size);
+			length, channel->rec_window_size);
 
 		  lsh_string_trunc(data, channel->rec_window_size);
 		}
 
-	      if (!data->length)
+	      if (!length)
 		{
 		  /* Ignore data packet */
 		  lsh_string_free(data);
 		  return;
 		}
-	      channel->rec_window_size -= data->length;
+	      channel->rec_window_size -= length;
 
 	      CHANNEL_RECEIVE(channel, CHANNEL_DATA, data);
 	    }
@@ -1255,7 +1266,7 @@ DEFINE_PACKET_HANDLER(static, channel_extended_data_handler,
   uint32_t type;
   struct lsh_string *data;
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_CHANNEL_EXTENDED_DATA)
@@ -1279,31 +1290,32 @@ DEFINE_PACKET_HANDLER(static, channel_extended_data_handler,
 	    }
 	  else
 	    {
-              if (data->length > channel->rec_max_packet)
+	      uint32_t length = lsh_string_length(data);
+              if (length > channel->rec_max_packet)
                 {
                   werror("Channel data larger than rec_max_packet. Extra data ignored.\n");
 		  lsh_string_trunc(data, channel->rec_max_packet);
                 }
 
-	      if (data->length > channel->rec_window_size)
+	      if (length > channel->rec_window_size)
 		{
 		  /* Truncate data to fit window */
 		  werror("Channel extended data overflow. "
 			 "Extra data ignored.\n");
 		  debug("   (data->length=%i, rec_window_size=%i).\n", 
-			data->length, channel->rec_window_size);
+			length, channel->rec_window_size);
 
 		  lsh_string_trunc(data, channel->rec_window_size);
 		}
 	      
-	      if (!data->length)
+	      if (!length)
 		{
 		  /* Ignore data packet */
 		  lsh_string_free(data);
 		  return;
 		}
 
-	      channel->rec_window_size -= data->length;
+	      channel->rec_window_size -= length;
 
 	      switch(type)
 		{
@@ -1335,7 +1347,7 @@ DEFINE_PACKET_HANDLER(static, channel_eof_handler,
   unsigned msg_number;
   uint32_t channel_number;
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_CHANNEL_EOF)
@@ -1396,7 +1408,7 @@ DEFINE_PACKET_HANDLER(static, channel_close_handler,
   unsigned msg_number;
   uint32_t channel_number;
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_CHANNEL_CLOSE)
@@ -1459,7 +1471,7 @@ DEFINE_PACKET_HANDLER(static, channel_open_confirm_handler,
   uint32_t window_size;
   uint32_t max_packet;
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_CHANNEL_OPEN_CONFIRMATION)
@@ -1513,7 +1525,7 @@ DEFINE_PACKET_HANDLER(static, channel_open_failure_handler,
   const uint8_t *language;
   uint32_t language_length;
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_CHANNEL_OPEN_FAILURE)
@@ -1555,7 +1567,7 @@ DEFINE_PACKET_HANDLER(static, channel_success_handler,
   uint32_t channel_number;
   struct ssh_channel *channel;
       
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_CHANNEL_SUCCESS)
@@ -1587,7 +1599,7 @@ DEFINE_PACKET_HANDLER(static, channel_failure_handler,
   uint32_t channel_number;
   struct ssh_channel *channel;
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_CHANNEL_FAILURE)
@@ -1614,17 +1626,12 @@ DEFINE_PACKET_HANDLER(static, channel_failure_handler,
     PROTOCOL_ERROR(connection->e, "Invalid CHANNEL_FAILURE message.");
 }
 
+/* During keyexchange, and when the connection's write buffer gets
+   full, we do not allow more channel data. This function is called
+   when data is reallowed. */
 static void
-do_channels_after_keyexchange(struct command_continuation *s UNUSED,
-			      struct lsh_object *x);
-
-static struct command_continuation
-channels_after_keyexchange =
-  { STATIC_HEADER, do_channels_after_keyexchange };
-    
-static void
-do_channels_after_keyexchange(struct command_continuation *s UNUSED,
-			      struct lsh_object *x)
+do_channels_wakeup(struct command_continuation *s UNUSED,
+		   struct lsh_object *x)
 {
   CAST(ssh_connection, connection, x);
   struct channel_table *table = connection->table;
@@ -1641,11 +1648,12 @@ do_channels_after_keyexchange(struct command_continuation *s UNUSED,
 	  && channel->send_adjust)
 	CHANNEL_SEND_ADJUST(channel, 0);
     }
-
-  /* Reinstall hook */
-  connection_after_keyexchange(connection, &channels_after_keyexchange);
 }
 
+static struct command_continuation
+channels_wakeup =
+  { STATIC_HEADER, do_channels_wakeup };
+
 void
 init_connection_service(struct ssh_connection *connection)
 {
@@ -1659,7 +1667,7 @@ init_connection_service(struct ssh_connection *connection)
   connection_clear_timeout(connection);
 
   /* Unfreeze channels after key exchange */
-  connection_after_keyexchange(connection, &channels_after_keyexchange);
+  connection_wakeup(connection, &channels_wakeup);
   
   connection->dispatch[SSH_MSG_GLOBAL_REQUEST]
     = &global_request_handler;
@@ -1709,70 +1717,6 @@ DEFINE_COMMAND(connection_service_command)
   COMMAND_RETURN(c, connection);
 }
 
-/* FIXME: Exactly what is the difference between this function and
-   init_connection_service. */
-void
-init_login_service(struct ssh_connection *connection)
-{
-  struct channel_table *table = make_channel_table();
-  
-  debug("channel.c: do_connection_service\n");
-  
-  connection->table = table;
-
-  /* Cancel handshake timeout */
-  connection_clear_timeout(connection);
-
-  /* Unfreeze channels after key exchange */
-  connection_after_keyexchange(connection, &channels_after_keyexchange);
-  
-  connection->dispatch[SSH_MSG_CHANNEL_OPEN]
-    = &channel_open_handler;
-  connection->dispatch[SSH_MSG_CHANNEL_REQUEST]
-    = &channel_request_handler;
-  
-  connection->dispatch[SSH_MSG_CHANNEL_WINDOW_ADJUST]
-    = &window_adjust_handler;
-  connection->dispatch[SSH_MSG_CHANNEL_DATA]
-    = &channel_data_handler;
-  connection->dispatch[SSH_MSG_CHANNEL_EXTENDED_DATA]
-    = &channel_extended_data_handler;
-
-  connection->dispatch[SSH_MSG_CHANNEL_EOF]
-    = &channel_eof_handler;
-  connection->dispatch[SSH_MSG_CHANNEL_CLOSE]
-    = &channel_close_handler;
-
-  connection->dispatch[SSH_MSG_CHANNEL_OPEN_CONFIRMATION]
-    = &channel_open_confirm_handler;
-  connection->dispatch[SSH_MSG_CHANNEL_OPEN_FAILURE]
-    = &channel_open_failure_handler;
-  
-  connection->dispatch[SSH_MSG_CHANNEL_SUCCESS]
-    = &channel_success_handler;
-  connection->dispatch[SSH_MSG_CHANNEL_FAILURE]
-    = &channel_failure_handler;
-
-  connection->dispatch[SSH_MSG_REQUEST_SUCCESS]
-    = &global_success_handler;
-  connection->dispatch[SSH_MSG_REQUEST_FAILURE]
-    = &global_failure_handler;
-}
-
- 
-DEFINE_COMMAND(login_service_command)
-     (struct command *s UNUSED,
-      struct lsh_object *a,
-      struct command_continuation *c,
-      struct exception_handler *e UNUSED)
-{
-  CAST(ssh_connection, connection, a);
-
-  init_login_service(connection);
-
-  COMMAND_RETURN(c, connection);
-}
-
 struct lsh_string *
 format_channel_close(struct ssh_channel *channel)
 {
@@ -1793,7 +1737,7 @@ channel_close(struct ssh_channel *channel)
 
       channel->flags |= CHANNEL_SENT_CLOSE;
       
-      C_WRITE(channel->connection, format_channel_close(channel) );
+      connection_send(channel->connection, format_channel_close(channel));
 
       if (channel->flags & CHANNEL_RECEIVED_CLOSE)
 	EXCEPTION_RAISE(channel->e, &finish_exception);
@@ -1817,7 +1761,7 @@ channel_eof(struct ssh_channel *channel)
       verbose("Sending EOF on channel %i\n", channel->channel_number);
 
       channel->flags |= CHANNEL_SENT_EOF;
-      C_WRITE(channel->connection, format_channel_eof(channel) );
+      connection_send(channel->connection, format_channel_eof(channel) );
 
       if ( (channel->flags & CHANNEL_CLOSE_AT_EOF)
 	   && (channel->flags & (CHANNEL_RECEIVED_EOF | CHANNEL_NO_WAIT_FOR_EOF)) )
@@ -1859,9 +1803,10 @@ struct lsh_string *
 channel_transmit_data(struct ssh_channel *channel,
 		      struct lsh_string *data)
 {
-  assert(data->length <= channel->send_window_size);
-  assert(data->length <= channel->send_max_packet);
-  channel->send_window_size -= data->length;
+  uint32_t length = lsh_string_length(data);
+  assert(length <= channel->send_window_size);
+  assert(length <= channel->send_max_packet);
+  channel->send_window_size -= length;
   
   return ssh_format("%c%i%fS",
 		    SSH_MSG_CHANNEL_DATA,
@@ -1874,9 +1819,10 @@ channel_transmit_extended(struct ssh_channel *channel,
 			  uint32_t type,
 			  struct lsh_string *data)
 {
-  assert(data->length <= channel->send_window_size);
-  assert(data->length <= channel->send_max_packet);
-  channel->send_window_size -= data->length;
+  uint32_t length = lsh_string_length(data);
+  assert(length <= channel->send_window_size);
+  assert(length <= channel->send_max_packet);
+  channel->send_window_size -= length;
 
   return ssh_format("%c%i%i%fS",
 		    SSH_MSG_CHANNEL_EXTENDED_DATA,
@@ -1934,8 +1880,8 @@ do_channel_write(struct abstract_write *w,
 	channel_eof(closure->channel);
     }
   else
-    C_WRITE(closure->channel->connection,
-	    channel_transmit_data(closure->channel, packet) );
+    connection_send(closure->channel->connection,
+		    channel_transmit_data(closure->channel, packet));
 }
 
 static void
@@ -1952,10 +1898,10 @@ do_channel_write_extended(struct abstract_write *w,
 	channel_eof(closure->super.channel);
     }
   else
-    C_WRITE(closure->super.channel->connection,
-	    channel_transmit_extended(closure->super.channel,
-				      closure->type,
-				      packet));
+    connection_send(closure->super.channel->connection,
+		    channel_transmit_extended(closure->super.channel,
+					      closure->type,
+					      packet));
 }
 
 struct abstract_write *
@@ -2156,10 +2102,10 @@ format_channel_open(int type, uint32_t local_channel_number,
 
   packet = lsh_string_alloc(l1 + l2);
 
-  ssh_format_write(OPEN_FORMAT, l1, packet->data, OPEN_ARGS);
+  ssh_format_write(OPEN_FORMAT, packet, 0, OPEN_ARGS);
 
   va_start(args, format);
-  ssh_vformat_write(format, l2, packet->data+l1, args);
+  ssh_vformat_write(format, packet, l1, args);
   va_end(args);
 
   return packet;
@@ -2200,10 +2146,10 @@ format_channel_request(int type, struct ssh_channel *channel,
 
   packet = lsh_string_alloc(l1 + l2);
 
-  ssh_format_write(REQUEST_FORMAT, l1, packet->data, REQUEST_ARGS);
+  ssh_format_write(REQUEST_FORMAT, packet, 0, REQUEST_ARGS);
 
   va_start(args, format);
-  ssh_vformat_write(format, l2, packet->data+l1, args);
+  ssh_vformat_write(format, packet, l1, args);
   va_end(args);
 
   return packet;
@@ -2230,10 +2176,10 @@ format_global_request(int type, int want_reply,
 
   packet = lsh_string_alloc(l1 + l2);
 
-  ssh_format_write(REQUEST_FORMAT, l1, packet->data, REQUEST_ARGS);
+  ssh_format_write(REQUEST_FORMAT, packet, 0, REQUEST_ARGS);
 
   va_start(args, format);
-  ssh_vformat_write(format, l2, packet->data+l1, args);
+  ssh_vformat_write(format, packet, l1, args);
   va_end(args);
 
   return packet;
