@@ -59,13 +59,27 @@ do_request_service(struct command *s,
 
 struct command *make_request_service(int service);
 
+/* GABA:
+   (class
+     (name escape_callback)
+     (super lsh_callback)
+     (vars
+       (help . "const char *")))
+*/
+
+#define DEFINE_ESCAPE(name, help) \
+static void do_##name(struct lsh_callback *self); \
+struct escape_callback \
+name = { { STATIC_HEADER, do_##name }, help }; \
+static void do_##name(struct lsh_callback *self UNUSED)
+
 /* GABA:
    (class
      (name escape_info)
      (vars
        (escape . uint8_t)
        ; Handlers, indexed by character.
-       (dispatch array (object lsh_callback) "0x100")))
+       (dispatch array (object escape_callback) "0x100")))
 */
 
 struct escape_info *make_escape_info(uint8_t escape);
@@ -128,8 +142,8 @@ make_client_x11_display(const char *display, struct lsh_string *fake);
        (exit_code . "int *")
 
        (not . int)
-       (port . "char *")
-       (remote object address_info)
+       (port . "const char *")
+       (target . "const char *")
 
        (local_user . "char *")
        (user . "char *")
@@ -182,12 +196,6 @@ client_parse_forward_arg(char *arg,
 			 uint32_t *listen_port,
 			 struct address_info **target);
 
-extern struct command client_options2remote;
-#define OPTIONS2REMOTE (&client_options2remote.super)
-
-extern struct command client_options2actions;
-#define OPTIONS2ACTIONS (&client_options2actions.super)
-
 extern const struct argp client_argp;
 
 
