@@ -31,10 +31,9 @@
 
 #include "atoms.h"
 #include "command.h"
-#include "debug.h"
 #include "format.h"
 #include "lookup_verifier.h"
-
+#include "lsh_string.h"
 #include "srp.h"
 #include "ssh.h"
 #include "werror.h"
@@ -107,8 +106,9 @@ do_handle_dh_reply(struct packet_handler *c,
     algorithm = closure->hostkey_algorithm;
   
   res = VERIFY(v, algorithm,
-	       closure->dh.exchange_hash->length, closure->dh.exchange_hash->data,
-	       signature->length, signature->data);
+	       lsh_string_length(closure->dh.exchange_hash),
+	       lsh_string_data(closure->dh.exchange_hash),
+	       lsh_string_length(signature), lsh_string_data(signature));
   lsh_string_free(signature);
 
   if (!res)
@@ -157,7 +157,7 @@ do_init_client_dh(struct keyexchange_algorithm *c,
   dh->algorithms = algorithms;
   
   /* Send client's message */
-  C_WRITE_NOW(connection, dh_make_client_msg(&dh->dh));
+  connection_send_kex(connection, dh_make_client_msg(&dh->dh));
 
   /* Install handler */
   connection->dispatch[SSH_MSG_KEXDH_REPLY] = &dh->super;
@@ -266,7 +266,7 @@ do_handle_srp_reply(struct packet_handler *s,
   /* Ask for SRP password */
   passwd = INTERACT_READ_PASSWORD(self->srp->tty, MAX_PASSWD,
 				  ssh_format("SRP password for %lS: ",
-					     self->srp->name), 1);
+					     self->srp->name));
   if (!passwd)
     {
       lsh_string_free(salt);
@@ -291,7 +291,7 @@ do_handle_srp_reply(struct packet_handler *s,
 		     "SRP failure: Invalid public value from server.");
       return;
     }
-  C_WRITE_NOW(connection, response);
+  connection_send_kex(connection, response);
   
   connection->dispatch[SSH_MSG_KEXSRP_PROOF]
     = make_srp_client_proof_handler(self->srp);
@@ -337,7 +337,7 @@ do_init_client_srp(struct keyexchange_algorithm *s,
   srp->m2 = NULL;
   
   /* Send client's message */
-  C_WRITE_NOW(connection, srp_make_init_msg(&srp->dh, self->name));
+  connection_send_kex(connection, srp_make_init_msg(&srp->dh, self->name));
 
   /* Install handler */
   connection->dispatch[SSH_MSG_KEXSRP_REPLY] = make_srp_reply_handler(srp);
