@@ -32,6 +32,7 @@
 #include "charset.h"
 #include "command.h"
 #include "format.h"
+#include "lsh_string.h"
 #include "parse.h"
 #include "publickey_crypto.h"
 #include "ssh.h"
@@ -209,7 +210,7 @@ do_userauth_success(struct packet_handler *c,
 
   trace("client_userauth.c: do_userauth_success\n");
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_USERAUTH_SUCCESS)
@@ -292,7 +293,7 @@ do_userauth_failure(struct packet_handler *c,
   trace("client_userauth.c: do_userauth_failure\n");
   assert(self->state->current < self->state->next);
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_USERAUTH_FAILURE)
@@ -347,7 +348,7 @@ do_userauth_banner(struct packet_handler *self UNUSED,
   uint32_t language_length;
   const uint8_t *language;
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_USERAUTH_BANNER)
@@ -565,9 +566,9 @@ do_none_login(struct client_userauth_method *s UNUSED,
 {
   verbose("Requesting authentication using the `none' method.\n");
   
-  C_WRITE(connection,
-          format_userauth_none(userauth->username,
-                               userauth->service_name));
+  connection_send(connection,
+		  format_userauth_none(userauth->username,
+				       userauth->service_name));
   
   return make_client_none_state(e);
 }
@@ -606,12 +607,12 @@ send_password(struct client_password_state *state)
   struct lsh_string *passwd
     = INTERACT_READ_PASSWORD(state->tty, MAX_PASSWD,
 			     ssh_format("Password for %lS: ",
-					state->userauth->username), 1);
+					state->userauth->username));
 
   if (passwd)
     {
       /* Password empty? */
-      if (!passwd->length)
+      if (!lsh_string_length(passwd))
 	{
 	  /* NOTE: At least on some systems, the getpass function
 	   * sets the tty to raw mode, disabling ^C, ^D and the like.
@@ -624,11 +625,11 @@ send_password(struct client_password_state *state)
 
       verbose("Requesting authentication using the `password' method.\n");
       
-      C_WRITE(state->connection,
-	      format_userauth_password(state->userauth->username,
-				       state->userauth->service_name,
-				       local_to_utf8(passwd, 1),
-				       1));
+      connection_send(state->connection,
+		      format_userauth_password(state->userauth->username,
+					       state->userauth->service_name,
+					       local_to_utf8(passwd, 1),
+					       1));
     }
   else
     {
@@ -856,7 +857,7 @@ do_userauth_pk_ok(struct packet_handler *s,
   uint32_t keyblob_length;
   const uint8_t *keyblob;
 
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (parse_uint8(&buffer, &msg_number)
       && (msg_number == SSH_MSG_USERAUTH_PK_OK)
@@ -891,9 +892,11 @@ do_userauth_pk_ok(struct packet_handler *s,
 	  signed_data = ssh_format("%S%lS", connection->session_id, request);
 	  request = ssh_format("%flS%fS", 
 			       request, 
-			       SIGN(key->private, key->type, signed_data->length, signed_data->data));
+			       SIGN(key->private, key->type,
+				    lsh_string_length(signed_data),
+				    lsh_string_data(signed_data)));
 	  lsh_string_free(signed_data);
-	  C_WRITE(connection, request);
+	  connection_send(connection, request);
 	  self->state->pending++;
 	}
       else
@@ -943,10 +946,11 @@ do_publickey_login(struct client_userauth_method *s,
 	
 	/* NOTE: The PEER_USERAUTH_REQUEST_KLUDGE only applies to the
 	 * signed data. */
-	C_WRITE(connection, 
-		format_userauth_publickey_query(userauth->username,
-						userauth->service_name,
-						key->type, key->public));
+	connection_send(connection, 
+			format_userauth_publickey_query
+			  (userauth->username,
+			   userauth->service_name,
+			   key->type, key->public));
       }
     connection->dispatch[SSH_MSG_USERAUTH_PK_OK] = make_pk_ok_handler(state);
     return &state->super;
