@@ -25,7 +25,7 @@
 #define LSH_CONNECTION_H_INCLUDED
 
 #include "abstract_io.h"
-#include "abstract_compress.h"
+#include "compress.h"
 #include "queue.h"
 #include "resource.h"
 #include "randomness.h"
@@ -70,7 +70,11 @@ enum peer_flag
 
     /* Don't include the originator port in X11 channel open messages,
      * for compatibility with SSH Inc's ssh version 2.0.x */
-    PEER_X11_OPEN_KLUDGE         = 0x00000010
+    PEER_X11_OPEN_KLUDGE         = 0x00000010,
+
+    /* Sun's SSH version 1.0 sends an entire list of locales
+     * in the language field of its KEXINIT message. */
+    PEER_KEXINIT_LANGUAGE_KLUDGE = 0x00000020,    
   };
 
 /* State affecting incoming keyexchange packets */
@@ -161,22 +165,29 @@ do_##NAME(struct packet_handler *s UNUSED,		\
        ; Cleanup
        (resources object resource_list)
 
-       ; Connected peer
+       ; Connected peer and local
        ; FIXME: Perhaps this should be a sockaddr or some other object
        ; that facilitates reverse lookups?
-       (peer object address_info);
+       (peer object address_info)
+       (local object address_info)
 
        ; Keyexchange
        (kexinit object make_kexinit)
        
        ; Receiving
-       (rec_max_packet . UINT32)
+       (rec_max_packet . uint32_t)
        (rec_mac    object mac_instance)
        (rec_crypto object crypto_instance)
        (rec_compress object compress_instance)
 
        ; Sending 
-       (raw   object abstract_write)  ; Socket connected to the other end 
+       (socket object lsh_fd)    ; Socket connected to the other end 
+       ; When crossing the soft_limit we stop reading data on user
+       ; channels, and set hard_limit to the current size plus some
+       ; margin. When crossing the hard_limit, the connection is closed.
+       (soft_limit . uint32_t)
+       (hard_limit . uint32_t)
+
        (write_packet object abstract_write)  ; Where to send packets
        					     ; through the pipeline
 
@@ -200,13 +211,16 @@ do_##NAME(struct packet_handler *s UNUSED,		\
        
        ; For the key re-exchange logic
        (key_expire object resource)
-       (sent_data . UINT32)
+       (sent_data . uint32_t)
        
        ; Invoked at the end of keyexchange.
        ; Automatically reset to zero after each invocation.
        ; Gets the connection as argument.
        (keyexchange_done object command_continuation)
-              
+       ; Called when key(re)exchange is finished, and when our
+       ; write buffer shrinks anough to allow more channel data.
+       (wakeup object command_continuation)
+       
        (kexinits array (object kexinit) 2)
        (literal_kexinits array (string) 2)
 
@@ -221,33 +235,33 @@ do_##NAME(struct packet_handler *s UNUSED,		\
        ))
 */
 
-/* Processes the packet at once, passing it on to the write buffer. */
-/* FIXME: Could be replaced by a single function doing the
- * compress, encrypt, mac. */
-#define C_WRITE_NOW(c, s) A_WRITE((c)->write_packet, (s) )
-
-/* Same as C_WRITE_NOW, except that it queues packets while key
- * exchange is in progress. */
-#define C_WRITE(c, s) connection_send((c), (s))
-
 struct ssh_connection *
 make_ssh_connection(enum connection_flag flags,
 		    struct address_info *peer,
+		    struct address_info *local,
 		    const char *id_comment,
 		    struct exception_handler *e);
 
-void connection_init_io(struct ssh_connection *connection,
-			struct abstract_write *raw,
-			struct randomness *r);
-
+void
+connection_init_io(struct ssh_connection *connection,
+		   struct lsh_fd *socket);
 
 void
 connection_after_keyexchange(struct ssh_connection *self,
 			     struct command_continuation *c);
 
+void
+connection_wakeup(struct ssh_connection *self,
+		  struct command_continuation *c);
+
 struct lsh_callback *
 make_connection_close_handler(struct ssh_connection *c);
 
+/* Processes the packet at once, passing it on to the write buffer. */
+void
+connection_send_kex(struct ssh_connection *self,
+		    struct lsh_string *message);
+
 /* Sending ordinary (non keyexchange) packets */
 void
 connection_send(struct ssh_connection *self,
@@ -282,4 +296,20 @@ extern struct packet_handler connection_fail_handler;
 extern struct packet_handler connection_forward_handler;
 extern struct packet_handler connection_disconnect_handler;
 
+/* Implemented in write_packet.h */
+struct abstract_write *
+make_write_packet(struct ssh_connection *connection,
+		  struct randomness *random,
+		  struct abstract_write *next);
+
+/* Implemented in debug.c */
+struct abstract_write *
+make_packet_debug(struct abstract_write *continuation,
+		  struct lsh_string *prefix);
+
+void send_debug_message(struct ssh_connection *connection,
+			const char *msg, int always_display);
+
+extern struct packet_handler connection_debug_handler;
+
 #endif /* LSH_CONNECTION_H_INCLUDED */
