@@ -1,8 +1,6 @@
 /* debug.c
  *
- *
- *
- * $Id: debug.c,v 1.36 2002/03/13 15:40:37 nisse Exp $ */
+ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -23,11 +21,14 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "debug.h"
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "connection.h"
 #include "format.h"
 #include "parse.h"
+#include "lsh_string.h"
 #include "ssh.h"
 #include "xalloc.h"
 #include "werror.h"
@@ -48,11 +49,11 @@ do_debug(struct abstract_write *w,
 {
   CAST(packet_debug, closure, w);
 
-  if (!packet->length)
+  if (!lsh_string_length(packet))
     debug("DEBUG: %S empty packet\n", closure->prefix);
   else
     {
-      UINT8 type = packet->data[0];
+      uint8_t type = lsh_string_data(packet)[0];
       if (type == SSH_MSG_USERAUTH_REQUEST)
 	debug("DEBUG: %S %z *****\n",
 	      closure->prefix, packet_types[type]);
@@ -95,35 +96,19 @@ send_debug_message(struct ssh_connection *connection,
 		   const char *msg, int always_display)
 {
   /* Can be sent even during key exchange. */
-  C_WRITE_NOW(connection, make_debug_packet(msg, always_display));
+  connection_send_kex(connection, make_debug_packet(msg, always_display));
 }
 
-#if 0
-void
-send_debug(struct abstract_write *write, const char *msg, int always_display)
-{
-  if (debug_flag)
-    send_debug_message(write, msg, always_display);
-}
-
-void
-send_verbose(struct abstract_write *write, const char *msg, int always_display)
-{
-  if (verbose_flag)
-    send_debug_message(write, msg, always_display);
-}
-#endif
-
 DEFINE_PACKET_HANDLER(, connection_debug_handler, connection UNUSED, packet)
 {
   struct simple_buffer buffer;
   unsigned msg_number;
   unsigned always_display;
-  UINT32 length;
-  const UINT8 *msg;
+  uint32_t length;
+  const uint8_t *msg;
   int language;
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (!(parse_uint8(&buffer, &msg_number)
 	&& parse_uint8(&buffer, &always_display)
