@@ -21,18 +21,25 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+
+#include "nettle/bignum.h"
+
 #include "publickey_crypto.h"
 
 #include "connection.h"
 #include "crypto.h"
 #include "format.h"
+#include "lsh_string.h"
 #include "randomness.h"
 #include "ssh.h"
 #include "werror.h"
 #include "xalloc.h"
 
-#include <assert.h>
-
 void
 init_dh_instance(struct dh_method *m,
 		 struct dh_instance *self,
@@ -62,7 +69,7 @@ init_dh_instance(struct dh_method *m,
 		 c->versions[CONNECTION_SERVER],
 		 c->literal_kexinits[CONNECTION_CLIENT],
 		 c->literal_kexinits[CONNECTION_SERVER]);
-  hash_update(self->hash, s->length, s->data);
+  hash_update(self->hash, STRING_LD(s));
 
   lsh_string_free(s);  
 }
@@ -100,7 +107,7 @@ dh_generate_secret(struct dh_method *self,
   /* Generate a random number, 1 < x <= p-1 = O(G) */
   mpz_init_set(tmp, self->G->order);  
   mpz_sub_ui(tmp, tmp, 1);
-  bignum_random(r, self->random, tmp);
+  nettle_mpz_random(r, self->random, lsh_random, tmp);
   mpz_add_ui(r, r, 1);
   mpz_clear(tmp);
 
@@ -122,7 +129,7 @@ dh_process_client_msg(struct dh_instance *self,
   unsigned msg_number;
   mpz_t tmp;
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (! (parse_uint8(&buffer, &msg_number)
 	 && (msg_number == SSH_MSG_KEXDH_INIT)
@@ -148,7 +155,7 @@ dh_hash_update(struct dh_instance *self,
 {
   debug("dh_hash_update: %xS\n", s);
   
-  hash_update(self->hash, s->length, s->data);
+  hash_update(self->hash, STRING_LD(s));
   if (free)
     lsh_string_free(s);
 }
@@ -160,8 +167,7 @@ dh_hash_digest(struct dh_instance *self)
   dh_hash_update(self, ssh_format("%n%n%S",
 				  self->e, self->f,
 				  self->K), 1);
-  self->exchange_hash = lsh_string_alloc(HASH_SIZE(self->hash));
-  hash_digest(self->hash, self->exchange_hash->data);
+  self->exchange_hash = hash_digest_string(self->hash);
 
   debug("dh_hash_digest: %xS\n", self->exchange_hash);  
 }
@@ -185,8 +191,8 @@ dh_make_server_msg(struct dh_instance *self,
 		    SSH_MSG_KEXDH_REPLY,
 		    server_key,
 		    self->f, SIGN(s, hostkey_algorithm,
-				  self->exchange_hash->length,
-				  self->exchange_hash->data));
+				  lsh_string_length(self->exchange_hash),
+				  lsh_string_data(self->exchange_hash)));
 }
 
 /* Returns the host key. */
@@ -202,7 +208,7 @@ dh_process_server_msg(struct dh_instance *self,
   struct lsh_string *key = NULL;
   struct lsh_string *s = NULL;
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (! (parse_uint8(&buffer, &msg_number)
 	 && (msg_number == SSH_MSG_KEXDH_REPLY)
