@@ -1,6 +1,5 @@
 /* gateway_commands.c
  *
- * $Id: gateway_commands.c,v 1.17 2002/08/25 08:17:09 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -22,77 +21,170 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "gateway_commands.h"
-
-#include "channel.h"
-#include "connection_commands.h"
-#include "debug.h"
-#include "format.h"
-#include "gateway_channel.h"
-#include "io_commands.h"
-#include "read_packet.h"
-#include "ssh.h"
-#include "unpad.h"
-#include "werror.h"
-#include "xalloc.h"
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <assert.h>
 #include <string.h>
 
+#include "nettle/macros.h"
+
+#include "gateway_commands.h"
+
+#include "channel.h"
+#include "connection_commands.h"
+#include "format.h"
+#include "gateway_channel.h"
+#include "io_commands.h"
+#include "lsh_string.h"
+#include "read_packet.h"
+#include "ssh.h"
+#include "werror.h"
+#include "xalloc.h"
+
+#define HEADER_SIZE 4
 #include "gateway_commands.c.x"
 
-/* A simplified version of do_pad in pad.c. This one uses
- * a fixed block size and pads with zeros. */
-
-/* FIXME: It seems very unnecessary to pad at all; we could just use
+/* The protocol used over the gateway socket is clear text ssh-packets
  *
  *   uint32     packet_length
  *   byte[n]    payload; n = packet_length
  *
- * But then we can't reuse the plain read_packet class. */
+ * There is no extra padding. Max packet size
+ * connection->rec_max_packet.*/
 
 static void
 do_gateway_pad(struct abstract_write *w,
 	       struct lsh_string *packet)
 {
-  CAST(abstract_write_pipe, closure, w);
+  CAST(abstract_write_pipe, self, w);
 
-  struct lsh_string *new;
-  
-  UINT32 new_size;
-  UINT8 padding;
-
-  UINT8 *data;
-
-  /* new_size is (packet->length + 9) rounded up to a multiple of
-   * block_size. But the block_size if fixed to 8 octets. */
-  new_size = 8 * (2 + packet->length / 8);
-
-  padding = new_size - packet->length - 5;
-  assert(padding >= 4);
-
-  new = ssh_format("%i%c%lr", packet->length + padding + 1,
-		   padding, packet->length + padding, &data);
-
-  assert(new->length == new_size);
-
-  memcpy(data, packet->data, packet->length);
-  memset(data + packet->length, '\0', padding);
-  
-  lsh_string_free(packet);
-
-  A_WRITE(closure->next, new);
+  A_WRITE(self->next, ssh_format("%fS", packet));
 }
 
 static struct abstract_write *
 make_gateway_pad(struct abstract_write *next)
 {
-  NEW(abstract_write_pipe, closure);
+  NEW(abstract_write_pipe, self);
 
-  closure->super.write = do_gateway_pad;
-  closure->next = next;
+  self->super.write = do_gateway_pad;
+  self->next = next;
 
-  return &closure->super;
+  return &self->super;
+}
+
+/* GABA:
+   (class
+     (name read_gateway_packet)
+     (super read_handler)
+     (vars
+       ; Buffer index, used for all the buffers
+       (pos . uint32_t)
+       (header string)
+       (payload string)
+       (handler object abstract_write)
+       (connection object ssh_connection)))
+*/
+
+static uint32_t
+do_read_gateway(struct read_handler **h,
+		uint32_t available,
+		const uint8_t *data)
+{
+  CAST(read_gateway_packet, self, *h);
+
+  if (!available)
+    {
+      debug("read_gateway: Got EOF.\n");
+
+      if (self->payload)
+        EXCEPTION_RAISE(self->connection->e,
+                        make_protocol_exception(0, "Unexpected EOF"));
+      else
+        EXCEPTION_RAISE(self->connection->e, &finish_read_exception);
+
+      *h = NULL;
+      return 0;
+    }
+
+  if (!self->payload)
+    {
+      /* Get length field */	
+      uint32_t left;
+      uint32_t length;
+	
+      assert(self->pos < HEADER_SIZE);	
+      left = HEADER_SIZE - self->pos;
+
+      if (available < left)
+	{
+	  lsh_string_write(self->header, self->pos, available, data);
+	  self->pos += available;
+	  return available;
+	}
+
+      lsh_string_write(self->header, self->pos, left, data);
+
+      length = READ_UINT32(lsh_string_data(self->header));
+      if (length > self->connection->rec_max_packet)
+	{
+	  static const struct protocol_exception too_large =
+	    STATIC_PROTOCOL_EXCEPTION(SSH_DISCONNECT_PROTOCOL_ERROR,
+				      "Packet too large");
+	    
+	  werror("read_gateway: Receiving too large packet.\n"
+		 "  %i octets, limit is %i\n",
+		 length, self->connection->rec_max_packet);
+	    
+	  EXCEPTION_RAISE(self->connection->e, &too_large.super);
+	  *h = NULL;
+	}
+      else
+	{
+	  self->payload = lsh_string_alloc(length);
+	  self->pos = 0;
+	}
+
+      return left;
+    }
+  else
+    {
+      uint32_t left;
+      left = lsh_string_length(self->payload) - self->pos;
+
+      if (available < left)
+	{
+	  lsh_string_write(self->payload, self->pos, available, data);
+	  self->pos += available;
+	  return available;
+	}
+
+      lsh_string_write(self->payload, self->pos, left, data);
+
+      A_WRITE(self->handler, self->payload);
+      self->payload = NULL;
+      self->pos = 0;
+
+      return left;
+    }
+}
+
+static struct read_handler *
+make_read_gateway(struct abstract_write *handler,
+		  struct ssh_connection *connection)
+{
+  NEW(read_gateway_packet, self);
+  self->super.handler = do_read_gateway;
+
+  self->connection = connection;
+  self->handler = handler;
+
+  self->header = lsh_string_alloc(HEADER_SIZE);
+  self->pos = 0;
+  self->payload = NULL;
+
+  return &self->super;
 }
 
 
@@ -112,22 +204,21 @@ gateway_make_connection(struct listen_value *lv,
 			  lv->peer, lv->local, "gateway",
 			  make_exc_finish_read_handler(lv->fd, e, HANDLER_CONTEXT));
 
-  /* Instead of calling connection_init_io. */
-  connection->raw =
-    &io_read_write(lv->fd,
-		   make_buffered_read(
-		     BUF_SIZE,
-		     make_read_packet(
-		       make_packet_unpad(
-			 connection,
-			 make_packet_debug(&connection->super,
-				 	  ssh_format("%lz received", connection->debug_comment))),
-		       connection)),
+  connection_init_io
+    (connection,
+     io_read_write(lv->fd,
+		   make_buffered_read
+		     (BUF_SIZE,
+		      make_read_gateway
+		        (make_packet_debug(&connection->super,
+					   ssh_format("%lz received",
+						      connection->debug_comment)),
+			 connection)),
 		   BLOCK_SIZE,
-		   make_connection_close_handler(connection))->write_buffer->super;
+		   make_connection_close_handler(connection)));
   
   connection->write_packet
-    = make_packet_debug(make_gateway_pad(connection->raw),
+    = make_packet_debug(make_gateway_pad(&connection->socket->write_buffer->super),
 			ssh_format("%lz sent", connection->debug_comment));
 
   init_connection_service(connection);
@@ -199,18 +290,3 @@ make_gateway_setup(struct local_info *local)
 
   return res;
 }
-
-#if 0
-DEFINE_COMMAND(gateway_setup_command)
-     (struct command *s UNUSED,
-      struct lsh_object *a,
-      struct command_continuation *c,
-      struct exception_handler *e UNUSED)
-{
-  CAST(local, local_info, a);
-  CAST_SUBTYPE(command, res,
-	       gateway_setup(local));
-
-  COMMAND_RETURN(c, res);
-}
-#endif
