@@ -33,14 +33,12 @@
 #include "command.h"
 #include "compress.h"
 #include "connection.h"
-#include "debug.h"
 #include "format.h"
 #include "io.h"
 #include "keyexchange.h"
 #include "read_line.h"
 #include "read_packet.h"
 #include "ssh.h"
-#include "unpad.h"
 #include "version.h"
 #include "werror.h"
 #include "xalloc.h"
@@ -74,7 +72,7 @@ compat[] =
   };
     
 static enum peer_flag
-compat_peer_flags(uint32_t length, uint8_t *software)
+compat_peer_flags(uint32_t length, const uint8_t *software)
 {
   unsigned i;
   for (i = 0; compat[i].prefix; i++)
@@ -109,14 +107,14 @@ compat_peer_flags(uint32_t length, uint8_t *software)
        (fallback object ssh1_fallback)))
 */
 
-/* Returns -1 if the line is not the start of a SSH handshake,
- * 0 if the line appears to be an SSH handshake, but with bogus version fields,
- * or 1 if the line was parsed sucessfully. */
+/* Returns -1 if the line is not the start of a SSH handshake, 0 if
+ * the line appears to be an SSH handshake, but with bogus version
+ * fields, or 1 if the line was parsed sucessfully. */
 static int
-split_version_string(uint32_t length, uint8_t *line,
-		     uint32_t *protover_len, uint8_t **protover,
-		     uint32_t *swver_len, uint8_t **swver,
-		     uint32_t *comment_len, uint8_t **comment)
+split_version_string(uint32_t length, const uint8_t *line,
+		     uint32_t *protover_len, const uint8_t **protover,
+		     uint32_t *swver_len, const uint8_t **swver,
+		     uint32_t *comment_len, const uint8_t **comment)
 {
   uint8_t *sep;
 
@@ -160,12 +158,12 @@ static void
 do_line(struct line_handler **h,
 	struct read_handler **r,
 	uint32_t length,
-	uint8_t *line,
+	const uint8_t *line,
 	struct exception_handler *e)
 {
   CAST(connection_line_handler, closure, *h);
   uint32_t protover_len, swver_len, comment_len;
-  uint8_t *protover, *swver, *comment;
+  const uint8_t *protover, *swver, *comment;
 
   struct ssh_connection *connection = closure->connection;
   int mode = connection->flags & CONNECTION_MODE;
@@ -199,20 +197,12 @@ do_line(struct line_handler **h,
 
 	    connection->peer_flags = compat_peer_flags(swver_len, swver);
 
-	    new = 
-	      make_read_packet(
-		make_packet_unpad(
-		  connection,
-		  make_packet_inflate(
+	    new = make_read_packet(
 		    make_packet_debug(&connection->super,
 				      (connection->debug_comment
 				       ? ssh_format("%lz received", connection->debug_comment)
 				       : ssh_format("Received"))),
-		    connection
-		    )
-		  ),
-		connection
-		);
+		    connection);
 	    
 	    connection->versions[!mode]
 	      = ssh_format("%ls", length, line);
@@ -296,7 +286,8 @@ make_handshake_info(enum connection_flag flags,
 		    uint32_t block_size,
 		    struct randomness *r,
 		    struct alist *algorithms,
-		    struct ssh1_fallback *fallback)
+		    struct ssh1_fallback *fallback,
+		    struct lsh_string *banner_text)
 {
   NEW(handshake_info, self);
   self->flags = flags;
@@ -306,6 +297,7 @@ make_handshake_info(enum connection_flag flags,
   self->random = r;
   self->algorithms = algorithms;
   self->fallback = fallback;
+  self->banner_text = banner_text;
 
   return self;
 }
@@ -391,15 +383,20 @@ DEFINE_COMMAND4(handshake_command)
   
   connection_init_io
     (connection, 
-     &io_read_write(lv->fd,
-		    make_buffered_read
-		    (BUF_SIZE,
-		     make_connection_read_line(connection, 
-					       lv->fd->fd, info->fallback)),
-		    info->block_size,
-		    make_connection_close_handler(connection))
-     ->write_buffer->super,
-     info->random);
+     io_read_write(lv->fd,
+		   make_buffered_read
+		   (BUF_SIZE,
+		    make_connection_read_line(connection, 
+					      lv->fd->fd, info->fallback)),
+		   info->block_size,
+		   make_connection_close_handler(connection)));
+
+  connection->write_packet =
+    make_packet_debug(make_write_packet(connection, info->random,
+					&connection->socket->write_buffer->super),
+		      (info->debug_comment
+		       ? ssh_format("%lz sent", info->debug_comment)
+		       : ssh_format("Sent")));
 
   /* Install timeout. */
   connection_set_timeout(connection,
@@ -419,13 +416,19 @@ DEFINE_COMMAND4(handshake_command)
    * until the client's identification string is received. */
   if (info->fallback)
     {
-      A_WRITE(connection->raw,
+      A_WRITE(&connection->socket->write_buffer->super,
 	      ssh_format("%lS\n", version));
       return;
     }
 #endif /* WITH_SSH1_FALLBACK */
 
-  A_WRITE(connection->raw,
+  if (info->banner_text)
+    {
+      A_WRITE(&connection->socket->write_buffer->super, 
+	      ssh_format("%lS\r\n", info->banner_text));
+    }
+
+  A_WRITE(&connection->socket->write_buffer->super,
 	  ssh_format("%lS\r\n", version));
   
   send_kexinit(connection);
