@@ -1,6 +1,5 @@
 /* handshake.c
  *
- * $Id: handshake.h,v 1.5 2002/08/25 08:14:44 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -41,13 +40,14 @@
        ; CONNECTION_SERVER or CONNECTION_CLIENT
        (flags . "enum connection_flag")
        
-       (block_size . UINT32)
+       (block_size . uint32_t)
        (id_comment . "const char *")
        (debug_comment . "const char *")
 
        (random object randomness)
        (algorithms object alist)
-       
+
+       (banner_text string)
        ; Used only on the server
        (fallback object ssh1_fallback)))
 */
@@ -56,10 +56,11 @@ struct handshake_info *
 make_handshake_info(enum connection_flag flags,
 		    const char *id_comment,
 		    const char *debug_comment,
-		    UINT32 block_size,
+		    uint32_t block_size,
 		    struct randomness *r,
 		    struct alist *algorithms,
-		    struct ssh1_fallback *fallback);
+		    struct ssh1_fallback *fallback,
+		    struct lsh_string *banner_text);
 
 extern struct command_4 handshake_command;
 #define CONNECTION_HANDSHAKE (&handshake_command.super.super)
