@@ -33,11 +33,15 @@
 #include <netinet/in.h>
 
 #include "abstract_io.h"
+#include "queue.h"
 #include "resource.h"
 #include "write_buffer.h"
 
 enum io_type { IO_PTY = 1 };
 
+/* Max number of simultaneous connection attempts */
+#define CONNECT_ATTEMPTS_LIMIT 3
+
 #define GABA_DECLARE
 #include "io.h.x"
 #undef GABA_DECLARE
@@ -240,11 +244,6 @@ int blocking_read(int fd, struct read_handler *r);
 
 int get_portno(const char *service, const char *protocol);
 
-struct address_info *
-make_address_info_c(const char *host,
-		    const char *port,
-		    int def);
-
 struct address_info *
 make_address_info(struct lsh_string *host, 
 		  uint32_t port);
@@ -262,6 +261,11 @@ address_info2sockaddr(socklen_t *length,
 		      const int *preference,
 		      int lookup);
 
+unsigned
+io_resolv_address(const char *host, const char *service,
+		  unsigned default_port,
+		  struct addr_queue *q);
+
 /* Returns an exception, if anything went wrong */
 const struct exception *
 write_raw(int fd, uint32_t length, const uint8_t *data);
@@ -290,12 +294,35 @@ make_exc_finish_read_handler(struct lsh_fd *fd,
 			     struct exception_handler *parent,
 			     const char *context);
 
+/* GABA:
+   (class
+     (name connect_list_state)
+     (super resource)
+     (vars
+       (q struct addr_queue)
+       ;; Number of currently active fd:s
+       (nfds . unsigned)
+       (fds array (object lsh_fd) CONNECT_ATTEMPTS_LIMIT)))
+*/
+
+struct connect_list_state *
+make_connect_list_state(void);
+
+struct resource *
+io_connect_list(struct connect_list_state *remote,
+		struct command_continuation *c,
+		struct exception_handler *e);
+
+/* FIXME: Reorder arguments to put length first, for consistency? */
 struct lsh_fd *
 io_connect(struct sockaddr *remote,
 	   socklen_t remote_length,
-	   struct command_continuation *c,
+	   struct io_callback *c,
 	   struct exception_handler *e);
 
+struct io_callback *
+make_connect_callback(struct command_continuation *c);
+
 struct lsh_fd *
 io_bind_sockaddr(struct sockaddr *local,
 		 socklen_t length,
@@ -305,6 +332,11 @@ struct lsh_fd *
 io_listen(struct lsh_fd *fd,
 	  struct io_callback *callback);
 
+struct resource *
+io_listen_list(struct addr_queue *addresses,
+	       struct io_callback *callback,
+	       struct exception_handler *e);
+
 struct lsh_fd *
 io_bind_local(struct local_info *info,
 	      struct exception_handler *e);
@@ -352,10 +384,6 @@ io_write_file(const char *fname, int flags,
 	      struct lsh_callback *c,
 	      struct exception_handler *e);
 
-struct lsh_fd *
-io_read_file(const char *fname, 
-	     struct exception_handler *e);
-
 int
 lsh_make_pipe(int *fds);
 
