@@ -36,9 +36,9 @@
 #include "alist.h"
 #include "command.h"
 #include "connection.h"
-#include "debug.h"
 #include "format.h"
 #include "io.h"
+#include "lsh_string.h"
 #include "parse.h"
 #include "publickey_crypto.h"
 #include "ssh.h"
@@ -91,7 +91,7 @@ parse_kexinit(struct lsh_string *packet, enum peer_flag peer_flags)
   struct int_list *lists[NLISTS];
   int i;
   
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   if (!parse_uint8(&buffer, &msg_number)
       || (msg_number != SSH_MSG_KEXINIT) )
@@ -192,7 +192,7 @@ send_kexinit(struct ssh_connection *connection)
   connection->literal_kexinits[mode] = s; 
   connection_send_kex_start(connection);  
 
-  C_WRITE_NOW(connection, lsh_string_dup(s));
+  connection_send_kex(connection, lsh_string_dup(s));
 
   /* NOTE: This feature isn't fully implemented, as we won't tell
    * the selected key exchange method if the guess was "right". */
@@ -201,7 +201,7 @@ send_kexinit(struct ssh_connection *connection)
       s = kex->first_kex_packet;
       kex->first_kex_packet = NULL;
 
-      C_WRITE_NOW(connection, s);
+      connection_send_kex(connection, s);
     }
 }
 
@@ -403,7 +403,7 @@ kex_make_key(struct hash_instance *secret,
   
   struct lsh_string *key;
   struct hash_instance *hash;
-  uint8_t *digest;
+  struct lsh_string *digest;
   
   key = lsh_string_alloc(key_length);
 
@@ -413,20 +413,19 @@ kex_make_key(struct hash_instance *secret,
     return key;
   
   hash = hash_copy(secret);
-  digest = alloca(HASH_SIZE(hash));
 
   hash_update(hash, 1, tags + type); 
-  hash_update(hash, session_id->length, session_id->data);
-  hash_digest(hash, digest);
+  hash_update(hash, STRING_LD(session_id));
+  digest = hash_digest_string(hash);
 
   /* Is one digest large anough? */
   if (key_length <= HASH_SIZE(hash))
-    memcpy(key->data, digest, key_length);
+    lsh_string_write(key, 0, key_length, lsh_string_data(digest));
 
   else
     {
       unsigned left = key_length;
-      uint8_t *dst = key->data;
+      uint32_t pos = 0;
       
       KILL(hash);
       hash = hash_copy(secret);
@@ -441,35 +440,39 @@ kex_make_key(struct hash_instance *secret,
 	   * H(secret | K_1 | ... | K_{n-1}) */
 
 	  struct hash_instance *tmp;
+
+	  assert(pos + left == key_length);
 	  
 	  /* Append digest to the key data. */
-	  memcpy(dst, digest, HASH_SIZE(hash));
-	  dst += HASH_SIZE(hash);
+	  lsh_string_write_string(key, pos, digest);
+	  pos += HASH_SIZE(hash);
 	  left -= HASH_SIZE(hash);
 
 	  /* And to the hash state */
-	  hash_update(hash, HASH_SIZE(hash), digest);
-
+	  hash_update(hash, HASH_SIZE(hash), lsh_string_data(digest));
+	  lsh_string_free(digest);
+	  
 	  if (left <= HASH_SIZE(hash))
 	    break;
 	  
 	  /* Get a new digest, without disturbing the hash object (as
 	   * we'll need it again). We use another temporary hash for
 	   * extracting the digest. */
-
+	  
 	  tmp = hash_copy(hash);
-	  hash_digest(tmp, digest);
+	  digest = hash_digest_string(tmp);
 	  KILL(tmp);
 	}
 
       /* Get the final digest, and use some of it for the key. */
-      hash_digest(hash, digest);
-      memcpy(dst, digest, left);
+      digest = hash_digest_string(hash);
+      lsh_string_write(key, pos, left, lsh_string_data(digest));
     }
+
+  lsh_string_free(digest);
   KILL(hash);
 
-  debug("Expanded key: %xs",
-	key->length, key->data);
+  debug("Expanded key: %xS", key);
 
   return key;
 }
@@ -500,8 +503,8 @@ kex_make_encrypt(struct crypto_instance **c,
     iv = kex_make_key(secret, algorithm->iv_size,
 		      IV_TYPE(type), session_id);
   
-  *c = MAKE_ENCRYPT(algorithm, key->data,
-		    iv ? iv->data : NULL);
+  *c = MAKE_ENCRYPT(algorithm, lsh_string_data(key),
+		    iv ? lsh_string_data(iv) : NULL);
 
   lsh_string_free(key);
   lsh_string_free(iv);
@@ -535,7 +538,8 @@ kex_make_decrypt(struct crypto_instance **c,
     iv = kex_make_key(secret, algorithm->iv_size,
 		      IV_TYPE(type), session_id);
   
-  *c = MAKE_DECRYPT(algorithm, key->data, iv ? iv->data : NULL);
+  *c = MAKE_DECRYPT(algorithm, lsh_string_data(key),
+		    iv ? lsh_string_data(iv) : NULL);
 
   lsh_string_free(key);
   lsh_string_free(iv);
@@ -562,7 +566,7 @@ kex_make_mac(struct hash_instance *secret,
   key = kex_make_key(secret, algorithm->key_size,
 		     type, session_id);
 
-  mac = MAKE_MAC(algorithm, algorithm->key_size, key->data);
+  mac = MAKE_MAC(algorithm, algorithm->key_size, lsh_string_data(key));
 
   lsh_string_free(key);
   return mac;
@@ -654,7 +658,7 @@ do_handle_newkeys(struct packet_handler *c,
   struct simple_buffer buffer;
   unsigned msg_number;
 
-  simple_buffer_init(&buffer, packet->length, packet->data);
+  simple_buffer_init(&buffer, STRING_LD(packet));
 
   verbose("Received NEWKEYS. Key exchange finished.\n");
   
@@ -781,38 +785,6 @@ make_simple_kexinit(struct randomness *r,
   return &res->super;
 }
 
-
-/* FIXME: Move this to a separate file keyexchange_commands.c? */
-/* (kexinit_filter simple_kexinit alist)
- *
- * Destructively modifies the simple_kexinit to include only hostkey
- * algorithms that have keys in alist. */
-
-DEFINE_COMMAND2(kexinit_filter)
-     (struct command_2 *s UNUSED,
-      struct lsh_object *a1,
-      struct lsh_object *a2,
-      struct command_continuation *c,
-      struct exception_handler *e UNUSED)
-{
-  CAST(simple_kexinit, init, a1);
-  CAST_SUBTYPE(alist, keys, a2);
-
-  init->hostkey_algorithms
-    = filter_algorithms(keys, init->hostkey_algorithms);
-
-  if (!init->hostkey_algorithms)
-    {
-      werror("No hostkey algorithms advertised.\n");
-      init->hostkey_algorithms = make_int_list(1, ATOM_NONE, -1);
-    }
-
-  assert(LIST_LENGTH(init->hostkey_algorithms));
-
-  COMMAND_RETURN(c, init);
-}
-
-
 static int
 install_keys(struct object_list *algorithms,
 	     struct ssh_connection *connection,
@@ -879,7 +851,7 @@ kex_build_secret(const struct hash_algorithm *H,
   struct hash_instance *hash = make_hash(H);
   struct lsh_string *s = ssh_format("%S%lS", K, exchange_hash);
 
-  hash_update(hash, s->length, s->data);
+  hash_update(hash, STRING_LD(s));
   lsh_string_free(s);
   
   return hash;
@@ -900,7 +872,7 @@ keyexchange_finish(struct ssh_connection *connection,
    * newkeys message. */
 
   assert(connection->send_kex_only);
-  C_WRITE_NOW(connection, ssh_format("%c", SSH_MSG_NEWKEYS));
+  connection_send_kex(connection, ssh_format("%c", SSH_MSG_NEWKEYS));
   
   /* A hash instance initialized with the key, to be used for key
    * generation */
