@@ -27,6 +27,10 @@
 #include "config.h"
 #endif
 
+/* The zlib using code isn't used now */
+#undef WITH_ZLIB
+#define WITH_ZLIB 0
+
 #include <assert.h>
 #include <errno.h>
 #include <stdlib.h>
@@ -70,22 +74,23 @@
 
 
 #if WITH_ZLIB
+# if HAVE_ZLIB_H
+#  include <zlib.h>
+# endif
+#endif /* WITH_ZLIB */
 
-#if HAVE_ZLIB_H
-#include <zlib.h>
-#endif
+#include "nettle/yarrow.h"
 
 #include "environ.h"
 #include "format.h"
 #include "io.h"
 #include "lock_file.h"
+#include "lsh_string.h"
 #include "version.h"
 #include "werror.h"
 #include "xalloc.h"
 
-#include "nettle/yarrow.h"
-
-
+#if WITH_ZLIB
 /* FIXME: Duplicated in zlib.c */
 
 /* zlib memory functions */
@@ -100,7 +105,6 @@ zlib_free(void *opaque UNUSED, void *address)
 {
   lsh_space_free(address);
 }
-
 #endif /* WITH_ZLIB */
 
 #include "lsh-make-seed.c.x"
@@ -443,7 +447,7 @@ get_dev_mem(struct yarrow256_ctx *ctx, enum source_type source)
 
       close(fd);
     }
-  #endif
+#endif
 }
 
 /* List of commands based on Peter Gutmann's cryptlib,
