@@ -85,29 +85,11 @@ struct lsh_class
 
 #define CLASS(c) (c##_class)
 
-#if DEBUG_ALLOC
+/* string.h */
+struct lsh_string;
 
-struct lsh_string_header
-{
-  int magic; /* For a sentinel value */
-  /* Where/how the string was allocated */
-  const char *clue;
-  struct lsh_string *prev;
-  struct lsh_string *next;
-};
-
-#endif  /* DEBUG_ALLOC */
-
-struct lsh_string
-{
-#if DEBUG_ALLOC
-  struct lsh_string_header header;
-#endif
-  uint32_t sequence_number;
-  /* NOTE: The allocated size may be larger than the string length. */
-  uint32_t length; 
-  uint8_t data[1];
-};
+void
+lsh_string_free(const struct lsh_string *packet);
 
 /* Forward declarations of various structures */
 
@@ -123,6 +105,7 @@ struct command;
 struct command_2;
 struct command_3;
 struct command_4;
+struct catch_report_collect;
 
 /* connection.h */
 struct ssh_connection;
