@@ -21,10 +21,17 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+#include <locale.h>
+#include <unistd.h>
+
 #include "charset.h"
 #include "client.h"
 #include "connection.h"
-#include "debug.h"
 #include "format.h"
 #include "gateway.h"
 #include "gateway_commands.h"
@@ -35,10 +42,6 @@
 #include "werror.h"
 #include "xalloc.h"
 
-#include <assert.h>
-#include <locale.h>
-#include <unistd.h>
-
 #include "lsh_argp.h"
 
 /* Forward declarations */
@@ -90,9 +93,11 @@ DEFINE_COMMAND(options2info)
 /* GABA:
    (expr
      (name make_lshg_connect)
+     (params
+       (actions object object_list))
      (expr
        (lambda (options)
-         ((progn (options2actions options))
+         ((progn actions)
 	  (gateway_init
 	    (connect_local (options2info options)))))))
 */
@@ -137,7 +142,7 @@ do_lshg_send_ignore(struct command *s,
   CAST(ssh_connection, connection, x);
 
   /* Can be sent even during key exchange. */
-  C_WRITE_NOW(connection, ssh_format("%c%z", SSH_MSG_IGNORE, self->msg));
+  connection_send_kex(connection, ssh_format("%c%z", SSH_MSG_IGNORE, self->msg));
 }
 
 static struct command *
@@ -206,7 +211,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	  break;
 	}
       assert(self->super.user);
-      assert(self->super.remote);
+      assert(self->super.target);
       
       if (object_queue_is_empty(&self->super.actions))
 	{
@@ -216,7 +221,7 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 
       self->gateway = make_gateway_address(self->super.local_user,
 					   self->super.user,
-					   self->super.remote);
+					   self->super.target);
 
       if (!self->gateway)
 	{
@@ -323,7 +328,7 @@ main(int argc, char **argv, const char** envp)
 
   {
     CAST_SUBTYPE(command, lshg_connect,
-		 make_lshg_connect());
+		 make_lshg_connect(queue_to_list(&options->super.actions)));
 
     COMMAND_CALL(lshg_connect, options, &discard_continuation,
 		 make_lshg_exception_handler(&default_exception_handler,
