@@ -3,11 +3,12 @@
 #endif
 
 #include <assert.h>
-#include <inttypes.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 
+#include "nettle-types.h"
+
 #if 1
 # define BYTE_FORMAT "0x%02x"
 # define BYTE_COLUMNS 8
