@@ -28,6 +28,9 @@
 #include <assert.h>
 #include <string.h>
 
+#include "nettle/bignum.h"
+#include "nettle/macros.h"
+
 #include "parse.h"
 
 #include "format.h"
@@ -37,8 +40,6 @@
 
 #include "parse_macros.h"
 
-#include "nettle/bignum.h"
-
 void
 simple_buffer_init(struct simple_buffer *buffer,
 		   uint32_t capacity, const uint8_t *data)
@@ -77,6 +78,7 @@ parse_string(struct simple_buffer *buffer,
   return 1;
 }
 
+/* FIXME: Get rid of this memcpy? */
 int
 parse_octets(struct simple_buffer *buffer,
 	     uint32_t length, uint8_t *start)
