@@ -27,8 +27,7 @@
 #define LSH_PARSE_H_INCLUDED
 
 #include "atoms.h"
-#include "bignum.h"
-
+#include "nettle/bignum.h"
 
 /* Simple buffer
  * NOTE: All instances are allocated on the stack.
@@ -43,7 +42,7 @@ struct simple_buffer
 
 void
 simple_buffer_init(struct simple_buffer *buffer,
-		  uint32_t capacity, const uint8_t *data);
+		   uint32_t capacity, const uint8_t *data);
 
 /* Returns 1 on success, 0 on failure */
 int
