@@ -22,11 +22,17 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <assert.h>
+
 #include "charset.h"
 #include "connection.h"
-#include "dsa.h"
 #include "format.h"
 #include "lookup_verifier.h"
+#include "lsh_string.h"
 #include "parse.h"
 #include "publickey_crypto.h"
 #include "server_userauth.h"
@@ -34,8 +40,6 @@
 #include "werror.h"
 #include "xalloc.h"
 
-#include <assert.h>
-
 #include "server_publickey.c.x"
 
 /* GABA:
@@ -58,7 +62,7 @@ static void
 do_authenticate(struct userauth *s,
 		struct ssh_connection *connection,
 		struct lsh_string *username,
-		UINT32 service UNUSED, 
+		uint32_t service UNUSED, 
 		struct simple_buffer *args,
 		struct command_continuation *c,
 		struct exception_handler *e)
@@ -68,9 +72,9 @@ do_authenticate(struct userauth *s,
   struct lsh_string *keyblob = NULL;
   struct lookup_verifier *lookup;
   struct verifier *v;
-  const UINT8 *signature_blob;
-  UINT32 signature_length;
-  UINT32 signature_start = 0;
+  const uint8_t *signature_blob;
+  uint32_t signature_length;
+  uint32_t signature_start = 0;
   int algorithm;
   int check_key;
 
@@ -178,7 +182,7 @@ do_authenticate(struct userauth *s,
 
 	  lsh_string_free(keyblob); 
 	  if (VERIFY(v, algorithm,
-		     signed_data->length, signed_data->data,
+		     lsh_string_length(signed_data), lsh_string_data(signed_data),
 		     signature_length, signature_blob))
 	    {
 	      werror("publickey authentication for user %S succeeded.\n", username);
