@@ -1,6 +1,5 @@
 /* server_session.c
  *
- * $Id: server_session.c,v 1.97 2002/08/27 17:17:12 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -22,10 +21,27 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
+
+#include <errno.h>
+
+#include <signal.h>
+
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
+
+#include <sys/types.h>
+#include <sys/socket.h>
+
 #include "server_session.h"
 
 #include "channel_commands.h"
+#include "environ.h"
 #include "format.h"
+#include "lsh_string.h"
 #include "read_data.h"
 #include "reaper.h"
 #include "server_pty.h"
@@ -37,20 +53,6 @@
 #include "werror.h"
 #include "xalloc.h"
 
-#include <errno.h>
-
-/* For debug */
-
-/* #include <string.h> */
-#if HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#include <sys/types.h>
-#include <sys/socket.h>
-
-#include <signal.h>
-
 #include "server_session.c.x"
 
 
@@ -63,7 +65,7 @@
        ; User information
        ;; (user object lsh_user)
 
-       (initial_window . UINT32)
+       (initial_window . uint32_t)
 
        ; Resource to kill when the channel is closed. 
        (process object lsh_process)
@@ -113,7 +115,7 @@ do_receive(struct ssh_channel *c,
 /* We may send more data */
 static void
 do_send_adjust(struct ssh_channel *s,
-	       UINT32 i UNUSED)
+	       uint32_t i UNUSED)
 {
   CAST(server_session, session, s);
 
@@ -145,7 +147,7 @@ do_eof(struct ssh_channel *channel)
 }
 
 struct ssh_channel *
-make_server_session(UINT32 initial_window,
+make_server_session(uint32_t initial_window,
 		    struct alist *request_types)
 {
   NEW(server_session, self);
@@ -277,10 +279,10 @@ do_exit_shell(struct exit_callback *c, int signaled,
 	      signaled ? ATOM_EXIT_SIGNAL : ATOM_EXIT_STATUS,
 	      channel->channel_number);
       
-      C_WRITE(channel->connection,
-	      (signaled
-	       ? format_exit_signal(channel, core, value)
-	       : format_exit(channel, value)) );
+      connection_send(channel->connection,
+		      (signaled
+		       ? format_exit_signal(channel, core, value)
+		       : format_exit(channel, value)) );
 
       /* We want to close the channel as soon as all stdout and stderr
        * data has been sent. In particular, we don't wait for EOF from
@@ -531,14 +533,14 @@ init_spawn_info(struct spawn_info *info, struct server_session *session,
   /* FIXME: Set SSH_ORIGINAL_COMMAND */
   if (session->term)
     {
-      env[i].name ="TERM";
+      env[i].name = ENV_TERM;
       env[i].value = session->term;
       i++;
     }
 
   if (info->pty && info->pty->tty_name)
     {
-      env[i].name ="SSH_TTY";
+      env[i].name = ENV_SSH_TTY;
       env[i].value = info->pty->tty_name;
       i++;
     }
@@ -559,7 +561,7 @@ init_spawn_info(struct spawn_info *info, struct server_session *session,
 				     info->peer->ip,
 				     info->peer->port
 				     );
-      env[i].name ="SSH_CLIENT";
+      env[i].name = ENV_SSH_CLIENT;
       env[i].value = session->client;
 
       i++;
@@ -568,11 +570,11 @@ init_spawn_info(struct spawn_info *info, struct server_session *session,
 #if WITH_X11_FORWARD
   if (session->x11)
     {
-      env[i].name = "DISPLAY";
+      env[i].name = ENV_DISPLAY;
       env[i].value = session->x11->display;
       i++;
 
-      env[i].name = "XAUTHORITY";
+      env[i].name = ENV_XAUTHORITY;
       env[i].value = session->x11->xauthority;
       i++;
     }
@@ -611,7 +613,6 @@ DEFINE_CHANNEL_REQUEST(shell_request_handler)
   spawn.login = 1;
     
   if (spawn_process(session, channel->connection->user, &spawn))
-    /* NOTE: The return value is not used. */
     COMMAND_RETURN(c, channel);
   else
     {
@@ -633,8 +634,8 @@ DEFINE_CHANNEL_REQUEST(exec_request_handler)
   static const struct exception exec_request_failed =
     STATIC_EXCEPTION(EXC_CHANNEL_REQUEST, "Exec request failed");
   
-  UINT32 command_len;
-  const UINT8 *command;
+  uint32_t command_len;
+  const uint8_t *command;
 
   if (!(parse_string(args, &command_len, &command)
 	&& parse_eod(args)))
@@ -664,7 +665,6 @@ DEFINE_CHANNEL_REQUEST(exec_request_handler)
       args[1] = lsh_get_cstring(s);
 
       if (spawn_process(session, channel->connection->user, &spawn))
-	/* NOTE: The return value is not used. */
 	COMMAND_RETURN(c, channel);
       else
 	EXCEPTION_RAISE(e, &exec_request_failed);
@@ -695,7 +695,7 @@ DEFINE_CHANNEL_REQUEST(exec_request_handler)
 
 static const char *
 lookup_subsystem(struct subsystem_request *self,
-		 UINT32 length, const UINT8 *name)
+		 uint32_t length, const uint8_t *name)
 {
   unsigned i;
   if (memchr(name, 0, length))
@@ -725,8 +725,8 @@ do_spawn_subsystem(struct channel_request *s,
   static struct exception subsystem_request_failed =
     STATIC_EXCEPTION(EXC_CHANNEL_REQUEST, "Subsystem request failed");
 
-  const UINT8 *name;
-  UINT32 name_length;
+  const uint8_t *name;
+  uint32_t name_length;
 
   const char *program;
       
@@ -823,8 +823,8 @@ do_alloc_pty(struct channel_request *c UNUSED,
 	  session->pty = pty;
 	  remember_resource(channel->resources, &pty->super);
 
-	  verbose(" granted.\n");
-	  COMMAND_RETURN(s, NULL);
+	  verbose(" ... granted.\n");
+	  COMMAND_RETURN(s, channel);
 
 	  /* Success */
 	  return;
@@ -871,7 +871,7 @@ do_window_change_request(struct channel_request *c UNUSED,
       if (session->pty && session->in && session->in->super.alive
           && tty_setwinsize(session->in->fd, &dims))
         /* Success. Rely on the terminal driver sending SIGWINCH */
-        COMMAND_RETURN(s, NULL);
+        COMMAND_RETURN(s, channel);
       else
         EXCEPTION_RAISE(e, &winch_request_failed);
     }
@@ -900,11 +900,11 @@ do_x11_req(struct channel_request *s UNUSED,
   static struct exception x11_request_failed =
     STATIC_EXCEPTION(EXC_CHANNEL_REQUEST, "x11-req failed");
 
-  const UINT8 *protocol;
-  UINT32 protocol_length;
-  const UINT8 *cookie;
-  UINT32 cookie_length;
-  UINT32 screen;
+  const uint8_t *protocol;
+  uint32_t protocol_length;
+  const uint8_t *cookie;
+  uint32_t cookie_length;
+  uint32_t screen;
   unsigned single;
   
   CAST(server_session, session, channel);
