@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: commands.c,v 1.14 2003/02/23 18:23:54 nisse Exp $
+ * @(#) $Id: commands.c,v 1.15 2003/05/16 07:34:20 nisse Exp $
  *
  * commands.c
  */
@@ -24,6 +24,10 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include "commands.h"
 
 int ls_longlist = 1;
