@@ -23,18 +23,9 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "srp.h"
-
-#include "crypto.h"
-#include "environ.h"
-#include "format.h"
-#include "interact.h"
-#include "io.h"
-#include "randomness.h"
-#include "srp.h"
-#include "version.h"
-#include "werror.h"
-#include "xalloc.h"
+#if HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <assert.h>
 #include <errno.h>
@@ -47,6 +38,20 @@
 #include <unistd.h>
 #endif
 
+#include "srp.h"
+
+#include "crypto.h"
+#include "environ.h"
+#include "format.h"
+#include "interact.h"
+#include "io.h"
+#include "lsh_string.h"
+#include "randomness.h"
+#include "srp.h"
+#include "version.h"
+#include "werror.h"
+#include "xalloc.h"
+
 #define BLOCK_SIZE 2000
 #define SALT_SIZE 20
 
@@ -141,12 +146,14 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	
 	if (self->file)
 	  {
-	    fd = io_write_file(self->file->data,
+	    const char *cfile = lsh_get_cstring(self->file);
+	    fd = io_write_file(cfile,
 			       O_CREAT | O_EXCL | O_WRONLY,
 			       0600, BLOCK_SIZE,
 			       NULL, self->e);
 	    if (!fd)
-	      argp_failure(state, EXIT_FAILURE, errno, "Could not open '%s'.", self->file->data);
+	      argp_failure(state, EXIT_FAILURE, errno,
+			   "Could not open '%s'.", cfile);
 	  }
 	else
 	  {
@@ -163,12 +170,12 @@ main_argp_parser(int key, char *arg, struct argp_state *state)
 	  struct lsh_string *again;
 
 	  pw = INTERACT_READ_PASSWORD(self->tty, 500,
-				      ssh_format("Enter new SRP password: "), 1);
+				      ssh_format("Enter new SRP password: "));
 	  if (!pw)
 	    argp_failure(state, EXIT_FAILURE, 0, "Aborted.");
 
 	  again = INTERACT_READ_PASSWORD(self->tty, 500,
-					 ssh_format("Again: "), 1);
+					 ssh_format("Again: "));
 	  if (!again)
 	    argp_failure(state, EXIT_FAILURE, 0, "Aborted.");
 
@@ -210,9 +217,8 @@ srp_gen(struct srp_gen_options *options)
   struct lsh_string *res;
 
   /* NOTE: Allows random to be of bad quality */
-  salt = lsh_string_alloc(SALT_SIZE);
+  salt = lsh_string_random(options->r, SALT_SIZE);
 
-  RANDOM(options->r, salt->length, salt->data);
   name = ssh_format("%lz", options->name);
 
   /* FIXME: Leaks some strings. */
