@@ -19,12 +19,25 @@
 #define SSH_MSG_KEXINIT                20
 #define SSH_MSG_NEWKEYS                21
 
+#define SSH_FIRST_KEYEXCHANGE_SPECIFIC 30
 /* 30-49 Key exchange method specific (numbers can be reused for
  *       different authentication methods) */
 
 #define SSH_MSG_KEXDH_INIT             30
 #define SSH_MSG_KEXDH_REPLY            31
 
+/* SRP based key exchange (experimental) */
+
+#define SSH_MSG_KEXSRP_INIT            30
+#define SSH_MSG_KEXSRP_REPLY           31
+#define SSH_MSG_KEXSRP_PROOF           32
+
+/* Diffie-hellman with group negotiation (experimental) */
+
+#define SSH_MSG_KEX_DH_MOD_GROUPS      30
+#define SSH_MSG_KEX_DH_MOD_INIT        31
+#define SSH_MSG_KEX_DH_MOD_REPLY       32
+
 /* 50-59 User authentication generic */
 
 #define SSH_FIRST_USERAUTH_GENERIC 50
@@ -78,7 +91,10 @@
 #define SSH_DISCONNECT_HOST_NOT_ALLOWED_TO_CONNECT      1
 #define SSH_DISCONNECT_PROTOCOL_ERROR                   2
 #define SSH_DISCONNECT_KEY_EXCHANGE_FAILED              3
-#define SSH_DISCONNECT_HOST_AUTHENTICATION_FAILED       4
+
+/* Formely known as SSH_DISCONNECT_HOST_AUTHENTICATION_FAILED */
+#define SSH_DISCONNECT_RESERVED                         4
+
 #define SSH_DISCONNECT_MAC_ERROR                        5
 #define SSH_DISCONNECT_COMPRESSION_ERROR                6
 #define SSH_DISCONNECT_SERVICE_NOT_AVAILABLE            7
@@ -86,6 +102,10 @@
 #define SSH_DISCONNECT_HOST_KEY_NOT_VERIFIABLE          9
 #define SSH_DISCONNECT_CONNECTION_LOST                 10
 #define SSH_DISCONNECT_BY_APPLICATION                  11
+#define SSH_DISCONNECT_TOO_MANY_CONNECTIONS            12
+#define SSH_DISCONNECT_AUTH_CANCELLED_BY_USER          13
+#define SSH_DISCONNECT_NO_MORE_AUTH_METHODS_AVAILABLE  14
+#define SSH_DISCONNECT_ILLEGAL_USER_NAME               15
 
 /* Channels */
 #define SSH_OPEN_ADMINISTRATIVELY_PROHIBITED 1 
@@ -168,4 +188,10 @@
 /* Subtract this to get a reasonable max_packet value for a channel. */
 #define SSH_CHANNEL_MAX_PACKET_FUZZ 100
 
+/* Largest block size we support (NOTE: This is not a protocol limit) */
+#define SSH_MAX_BLOCK_SIZE 16
+
+/* Largest MAC digest size we support  (NOTE: This is not a protocol limit) */
+#define SSH_MAX_MAC_SIZE 20
+
 #endif /* LSH_SSH_H_INCLUDED */
