@@ -44,6 +44,7 @@
 
 #include "format.h"
 #include "io.h"
+#include "lsh_string.h"
 #include "resource.h"
 #include "suspend.h"
 #include "tty.h"
@@ -156,6 +157,7 @@ do_kill_window_subscriber(struct resource *s)
      (super interact)
      (vars
        (tty_fd . int)
+       (askpass . "const char *")
        ; Signal handler
        (winch_handler object resource)
        (nsubscribers . unsigned)
@@ -174,33 +176,89 @@ unix_is_tty(struct interact *s)
 
 /* FIXME: Rewrite to operate on tty_fd. */
 static struct lsh_string *
-unix_read_password(struct interact *s UNUSED,
+unix_read_password(struct interact *s,
 		   uint32_t max_length UNUSED,
-		   const struct lsh_string *prompt, int free)
+		   const struct lsh_string *prompt)
 {
-  /* NOTE: Ignores max_length; instead getpass's limit applies. */
+  CAST(unix_interact, self, s);
+  trace("unix_interact.c:unix_read_password\n");
   
-  char *password;
-  const char *cprompt = lsh_get_cstring(prompt);
-
-  if (!cprompt)
+  if (self->askpass)
     {
-      if (free)
-	lsh_string_free(prompt);
-      return NULL;
-    }
-  /* NOTE: This function uses a static buffer. */
-  password = getpass(cprompt);
+      const char *argv[3];
+      struct lsh_string *password;
+      
+      int null = open("/dev/null", O_RDONLY);
+      
+      if (null < 0)
+	{
+	  werror("Failed to open /dev/null!\n");
+	  lsh_string_free(prompt);
+	  
+	  return NULL;
+	}
+      
+      argv[0] = self->askpass;
+      argv[1] = lsh_get_cstring(prompt);
+      if (!argv[1])
+	{
+	  close(null);
+	  lsh_string_free(prompt);
 
-  if (free)
-    lsh_string_free(prompt);
+	  return NULL;
+	}
+      argv[2] = NULL;
+
+      trace("unix_interact.c: spawning askpass program `%z'\n",
+	    self->askpass);
+
+      password = lsh_popen_read(self->askpass, argv, null, 100);
+
+      lsh_string_free(prompt);
+      
+      return password;
+    }
+  else
+    {
+      /* NOTE: Ignores max_length; instead getpass's limit applies. */
   
-  if (!password)
-    return NULL;
+      char *password;
+      const char *cprompt;
+
+      if (!IS_TTY(self) || quiet_flag)
+	{
+	  lsh_string_free(prompt);
+	  return NULL;
+	}
+
+      cprompt = lsh_get_cstring(prompt);
+      if (!cprompt)
+	{
+	  lsh_string_free(prompt);
+	  return NULL;
+	}
+      /* NOTE: This function uses a static buffer. */
+      password = getpass(cprompt);
   
-  return make_string(password);
+      lsh_string_free(prompt);
+  
+      if (!password)
+	return NULL;
+  
+      return make_string(password);
+    }
 }
 
+static void
+unix_set_askpass(struct interact *s,
+		 const char *askpass)
+{
+  CAST(unix_interact, self, s);
+  trace("unix_interact.c:unix_set_askpass\n");
+  assert(askpass);
+  
+  self->askpass = askpass;
+}
 
 static int
 unix_yes_or_no(struct interact *s,
@@ -221,7 +279,7 @@ unix_yes_or_no(struct interact *s,
       uint8_t buffer[TTY_BUFSIZE];
       const struct exception *e;
   
-      e = write_raw(self->tty_fd, prompt->length, prompt->data);
+      e = write_raw(self->tty_fd, STRING_LD(prompt));
 
       if (free)
 	lsh_string_free(prompt);
@@ -403,6 +461,7 @@ make_unix_interact(void)
   
   self->super.is_tty = unix_is_tty;
   self->super.read_password = unix_read_password;
+  self->super.set_askpass = unix_set_askpass;
   self->super.yes_or_no = unix_yes_or_no;
   self->super.get_attributes = unix_get_attributes;
   self->super.set_attributes = unix_set_attributes;
@@ -410,6 +469,7 @@ make_unix_interact(void)
   self->super.window_change_subscribe = unix_window_change_subscribe;
 
   self->tty_fd = -1;
+  self->askpass = NULL;
   
 #if HAVE_STDTTY_FILENO
   if (isatty(STDTTY_FILENO))
